@@ -6,6 +6,7 @@ import socket
 from array import array
 import pprint
 from threading import Event, Thread
+from datetime import datetime
 
 from .proto import Method, Frame
 from .method_io import MethodReader, MethodWriter
@@ -40,7 +41,8 @@ class Connection(AbstractChannel):
     def __init__(self, *, host='localhost', port=5672, ssl=None, connect_timeout=None,
                  userid='guest', password='guest', login_method='AMQPLAIN', virtual_host='/',
                  locale='en_US',
-                 channel_max=65535, frame_max=131072, heartbeat=0, auto_heartbeat=False, ahbd=True,
+                 channel_max=65535, frame_max=131072,
+                 heartbeat=0, auto_heartbeat=False, daemon=True,
                  client_properties=None,
                  on_blocked=None, on_unblocked=None):
         """Create a connection to the specified host
@@ -60,7 +62,7 @@ class Connection(AbstractChannel):
         :param int channel_max: maximum number of channels
         :param int frame_max: maximum frame payload size in bytes
         :param float heartbeat: heartbeat interval in seconds, 0 disables heartbeat
-        :param bool ahbd: auto heartbeat thread daemon mode
+        :param bool daemon: auto heartbeat thread daemon mode
         :param bool auto_heartbeat: enable automatic heartbeats thread
         :param client_properties: dict of client properties
         :param on_blocked: callback on connection blocked
@@ -79,21 +81,11 @@ class Connection(AbstractChannel):
         # the connection object itself is treated as channel 0
         super().__init__(self, 0)
 
-        # properties set in the Tune method
-        self.channel_max = channel_max
-        self.frame_max = frame_max
-        # final heartbeat interval value (in float seconds) after negotiation
-        self.heartbeat = 0
-        # original heartbeat interval value proposed by client
-        self.client_heartbeat = heartbeat
-        # original heartbeat interval proposed by server
-        self.server_heartbeat = None
-
-        # callbacks
-        self.on_blocked = on_blocked
-        self.on_unblocked = on_unblocked
-
-        self._avail_channel_ids = array('H', range(self.channel_max, 0, -1))
+        ## instance variables
+        self.transport = None
+        self.method_reader = None
+        self.method_writer = None
+        self._wait_tune_ok = None
 
         # properties set in the start method, after a connection is established
         self.version_major = 0
@@ -102,8 +94,51 @@ class Connection(AbstractChannel):
         self.mechanisms = []
         self.locales = []
 
+        # properties set in the Tune method
+        self.channel_max = channel_max
+        self.frame_max = frame_max
+        self._avail_channel_ids = array('H', range(self.channel_max, 0, -1))
+        self._heartbeat_final = 0  # final heartbeat interval after negotiation
+        self._heartbeat_server = None
+
+        # save connection parameters
+        self._host = host
+        self._port = port
+        self._connect_timeout = connect_timeout
+        self._ssl = ssl
+        self._userid = userid
+        self._password = password
+        self._login_method = login_method
+        self._virtual_host = virtual_host
+        self._locale = locale
+        self._heartbeat_client = heartbeat  # original heartbeat interval value proposed by client
+        self._auto_heartbeat = auto_heartbeat
+        self._daemon = daemon
+        self._client_properties = client_properties
+
+        ## callbacks
+        self.on_blocked = on_blocked
+        self.on_unblocked = on_unblocked
+
+        ## heartbeat
+        self.last_heartbeat_sent = None
+        self._close_event = None  # Event(): the heartbeat thread listens for this event to exit
+        self.heartbeat_thread = None
+
+        self.connect()
+
+    def connect(self):
+        """Connect using saved connection parameters
+
+        This method does not need to be called explicitly; it is called by the constructor during
+        initialization.
+
+        Note: reconnecting invalidates all declarations (channels, queues, consumers, delivery tags,
+        etc.).
+        """
         # start the connection; this also sends the connection protocol header
-        self.transport = create_transport(host, port, connect_timeout, frame_max, ssl)
+        self.transport = create_transport(self._host, self._port, self._connect_timeout,
+                                          self.frame_max, self._ssl)
 
         # create global instances of `MethodReader` and `MethodWriter` which can be used by all
         # channels
@@ -115,29 +150,32 @@ class Connection(AbstractChannel):
 
         # create 'login response' to send to server
         login_response = AMQPWriter()
-        login_response.write_table({'LOGIN': userid, 'PASSWORD': password})
+        login_response.write_table({'LOGIN': self._userid, 'PASSWORD': self._password})
         login_response = login_response.getvalue()[4:]  # skip the length
 
         # reply with 'start-ok' and connection parameters
         # noinspection PyArgumentList
-        client_props = dict(LIBRARY_PROPERTIES, **client_properties or {})
-        self._send_start_ok(client_props, login_method, login_response, locale)
+        client_props = dict(LIBRARY_PROPERTIES, **self._client_properties or {})
+        self._send_start_ok(client_props, self._login_method, login_response, self._locale)
 
         self._wait_tune_ok = True
         while self._wait_tune_ok:
             self.wait_any([spec.Connection.Secure, spec.Connection.Tune])
 
-        self._send_open(virtual_host)
+        self._send_open(self._virtual_host)
 
         # set up automatic heartbeats
         self._close_event = Event()
-        self.heartbeat_thread = None
-        if auto_heartbeat:
+        if self._auto_heartbeat:
             log.debug('Start automatic heartbeat thread')
-            t = Thread(target=self._heartbeat_thread, name='HeartbeatThread', daemon=ahbd)
+            t = Thread(target=self._heartbeat_thread, name='HeartbeatThread', daemon=self._daemon)
             t.start()
             self.heartbeat_thread = t
 
+    @property
+    def last_heartbeat_recv(self):
+        return self.method_reader.last_heartbeat_recv
+
     @property
     def connected(self):
         """Check if connection is connected
@@ -182,6 +220,7 @@ class Connection(AbstractChannel):
     def send_heartbeat(self):
         """Send a heartbeat to the server
         """
+        self.last_heartbeat_sent = datetime.now()
         self.transport.write_frame(Frame(FrameType.HEARTBEAT))
 
     def is_alive(self):
@@ -295,7 +334,7 @@ class Connection(AbstractChannel):
         # `is_alive()` sends heartbeats if the connection is alive
         while self.is_alive():
             # `close` is set to true if the `close_event` is signalled
-            close = self._close_event.wait(self.heartbeat / 1.5)
+            close = self._close_event.wait(self._heartbeat_final / 1.5)
             if close:
                 break
 
@@ -579,29 +618,29 @@ class Connection(AbstractChannel):
                 not want a heartbeat.
         """
         args = method.args
-        client_heartbeat = self.client_heartbeat or 0
+        client_heartbeat = self._heartbeat_client or 0
         # maximum number of channels that the server supports
         self.channel_max = min(args.read_short(), self.channel_max)
         # largest frame size the server proposes for the connection
         self.frame_max = min(args.read_long(), self.frame_max)
         self.method_writer.frame_max = self.frame_max
         # heartbeat interval proposed by server
-        self.server_heartbeat = args.read_short() or 0
+        self._heartbeat_server = args.read_short() or 0
 
         # negotiate the heartbeat interval to the smaller of the specified values
-        if self.server_heartbeat == 0 or client_heartbeat == 0:
-            self.heartbeat = max(self.server_heartbeat, client_heartbeat)
+        if self._heartbeat_server == 0 or client_heartbeat == 0:
+            self._heartbeat_final = max(self._heartbeat_server, client_heartbeat)
         else:
-            self.heartbeat = min(self.server_heartbeat, client_heartbeat)
+            self._heartbeat_final = min(self._heartbeat_server, client_heartbeat)
 
         # Ignore server heartbeat if client_heartbeat is disabled
-        if not self.client_heartbeat:
-            self.heartbeat = 0
+        if not self._heartbeat_client:
+            self._heartbeat_final = 0
 
-        self._send_tune_ok(self.channel_max, self.frame_max, self.heartbeat)
+        self._send_tune_ok(self.channel_max, self.frame_max, self._heartbeat_final)
 
     def _send_tune_ok(self, channel_max, frame_max, heartbeat):
-        """Negotiate cnnection tuning parameters
+        """Negotiate connection tuning parameters
 
         This method sends the client's connection tuning parameters to the server. Certain fields
         are negotiated, others provide capability information.
