@@ -3,6 +3,7 @@ from threading import Lock
 from collections import defaultdict, deque
 from queue import Queue
 import logging
+from datetime import datetime
 
 from .concurrency import synchronized
 from .exceptions import UnexpectedFrame, Timeout, METHOD_NAME_MAP
@@ -51,6 +52,7 @@ class MethodReader:
 
         self.heartbeats = 0  # total number of heartbeats received
         self.frames_recv = 0  # total number of frames received
+        self.last_heartbeat_recv = None
 
         self._method_read_lock = Lock()
 
@@ -87,6 +89,7 @@ class MethodReader:
 
     def _process_heartbeat(self):
         self.heartbeats += 1
+        self.last_heartbeat_recv = datetime.now()
 
     def _process_method_frame(self, frame):
         """Process method frame
